#include "stdafx.h"

char * szReportClass = "QUBReportWindow" ;

//-----------------------------------------------------------------------------
const int ROW_HGT = 20;			//-- size of message display 
const int QUE_SIZE = 100;		//-- size of message queue

HWND g_hWndReport=NULL;			//-- the global report window 
CItemArray g_aMsg;				//-- Array of messages 

//-----------------------------------------------------------------------------
// Class for each report message object 
class CReportMessage : public CItem { 
	public :
		CReportMessage( COLORREF crBgr, const char * pchMessage ) { 
			m_crBgr=crBgr;
			m_cWhen.Set(CurrentTime(true,true).Buffer());
			m_cMsg.Set(pchMessage);
			}
		COLORREF m_crBgr;
		CString m_cWhen;
		CString m_cMsg;
	};
	

//-----------------------------------------------------------------------------
//-- Message window functions 

void MoveMessageWindow( int iX, int iY, int iWid, int iHgt ) { 
	SetWindowPos( g_hWndReport, NULL, iX, iY, iWid, iHgt, SWP_NOZORDER );
	}

void AddMessage( COLORREF crBgr, const char * pchMessage ) {
	//-- keep this func thread callable - No UI code.
	g_aMsg.AddItem( new CReportMessage( crBgr, pchMessage ) );
	InvalidateRect( g_hWndReport, NULL, true );
	}

//-- Add an error message to the report window 
void ErrorMessage( int iMsgID, int iCode, const char * pchInfo ) { 
	//-- build error message 
	char buffer[1000];
	LoadString( GetInstance(), iMsgID, buffer, 200 );
	sprintf( buffer+strlen(buffer), " msg=%d code=%d ", iMsgID, iCode );
	if( pchInfo != NULL ) { 
		int iLen = strlen(buffer);
		if( iLen+strlen(pchInfo)+2 <= 1000 ) 
			sprintf( buffer+iLen, "{%s}", pchInfo );
		}
	strcpy( buffer+strlen(buffer), "\n" );
	//-- copy to display queue and debugger windows
	#ifdef _DEBUG
	OutputDebugString( buffer );
	#endif
	AddMessage( REPTYPE_ERROR, buffer );
	}

//-- create the report window if it does not exist 
void CreateReportWindow( HWND hPar ) { 
	if( g_hWndReport == NULL ) { 
		g_hWndReport = CreateMDIWindow( szReportClass, "Message Window", NULL, 
							0, 0, 500, 160, hPar, GetInstance(), 0 );
		ShowWindow( g_hWndReport, SW_SHOW );
		}
	}

//-- Window class registration
void RegisterReportClass( ) { 
	CreateWindowClass( szReportClass, (WNDPROC)ReportProc, GetInstance(), (LPCTSTR)IDI_REPORTWIN, (LPCTSTR)IDI_SMALL, sizeof(HANDLE), COLOR_WINDOW );
	}

//-- Message handler 
LRESULT CALLBACK ReportProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam){
	switch (message){
		case WM_PAINT: {
			// TODO - only redraw messages in the invalidated region 
			PAINTSTRUCT ps;
			RECT rct;
			HDC hdc = BeginPaint(hWnd, &ps);
			GetWindowRect( hWnd, &rct );
			int iPos=0;
			int iHgt=rct.bottom-rct.top;
			for( int i=g_aMsg.Count()-1; i>=0 && iPos<=iHgt; --i ) { 
				CReportMessage * pMsg = (CReportMessage *) g_aMsg[i];

				SetRect( &rct, 5, iPos, 140, iPos+ROW_HGT );
				ExtTextOut( hdc, 5, iPos, ETO_CLIPPED, &rct, pMsg->m_cWhen.Buffer(), strlen(pMsg->m_cWhen.Buffer()), NULL);
				
				ExtTextOut( hdc, 145, iPos, 0, NULL, pMsg->m_cMsg.Buffer(), strlen(pMsg->m_cMsg.Buffer()), NULL);
				iPos+=ROW_HGT;
				}
			EndPaint(hWnd, &ps);
			}
			break;
		
		case WM_DESTROY : 
			g_hWndReport=NULL;
			break;
			
		case WM_SIZE :
			InvalidateRect( g_hWndReport, NULL, true );
			break;
		}

	return DefMDIChildProc( hWnd, message, wParam, lParam);
	}
